<?php

if(!defined('ABSPATH')) exit; /* Exit if accessed directly */

class wp_linkbuilder {
 
 //Plugin generics
 public function __construct($pp_plugin_name, $pp_plugin_name_read, $pp_plugin_filefolder_name = false) {
  //Set vars
  $this->pp_plugin_name = $pp_plugin_name;
  
  if($pp_plugin_filefolder_name == false) {
   //Same as normal name
   $this->pp_plugin_filefolder_name = $pp_plugin_name;
   } else {
   //Different name
   $this->pp_plugin_filefolder_name = $pp_plugin_filefolder_name;
   }
   
  $this->pp_prefix = $pp_plugin_name.'_';
  $this->pp_plugin_name_read = $pp_plugin_name_read;
  
  $this->pp_config['enable_backlink_promotion'] = 'N';
  $this->pp_config['auto_backlink_promotion'] = 'Y';
  $this->pp_config['show_disabled_pro_features'] = 'Y';
  $this->pp_config['enable_private_redirection'] = 'N';
  $this->pp_config['promotion_timeout'] = '5';
  
  //Non configurable constants
  $this->pp_constant['private_redirection_api_endpoint'] = 'http://www.wplinkbuilder.net/redirection/';
  $this->pp_constant['private_redirection_api_name'] = 'pp_linkbuilder_server_redirector';
  $this->pp_constant['promotion_api_endpoint'] = 'https://api.wplinkbuilder.net/wp-admin/admin-ajax.php';
  $this->pp_constant['promotion_api_name'] = 'pp_linkbuilder_server_promoter';
  }
 
 //Generic stuff
 public function activate() {
  /* Description: Performs activation */
  $plugin_data = $this->getPluginData($this->pp_plugin_name_wp); //because of wp naming conventions it cannot be wp_linkbuilder, but is wp-linkbuilder...
   
  //Add generic options
  add_option($this->pp_prefix.'version', $plugin_data['Version']);
  add_option($this->pp_prefix.'db_version', $plugin_data['Version']);
  
  //Always update these options
  update_option($this->pp_prefix.'db_version', $plugin_data['Version']);
  
  //Create tables
  global $wpdb;
  require_once(ABSPATH.'wp-admin/includes/upgrade.php');
  
  dbDelta("CREATE TABLE ".$wpdb->prefix.$this->pp_prefix."backlink (
				       backlink_id bigint(20) NOT NULL AUTO_INCREMENT,
				       url_referrer varchar(250) NOT NULL DEFAULT '0.0.0.0',
				       url_linkto varchar(250) NOT NULL DEFAULT '0.0.0.0',
				       date_detected datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
				       status_promoted enum ('Y','N') NOT NULL DEFAULT 'N',
				       PRIMARY KEY  (backlink_id)
				       ) ".$wpdb->get_charset_collate().";");
	 }
  
 function deactivate() {
  /* Description: Performs deactivation */
  }
 
 function uninstall() {
  /* Description: Performs uninstallation */
  //Delete options
  delete_option($this->pp_prefix.'version');
  delete_option($this->pp_prefix.'db_version');
  
  //Delete default options
  foreach($this->pp_config AS $key => $value) {
   $this->pp_config[$key] = delete_option($this->pp_prefix.$key);
   }
  
  //Delete tables
  $wpdb->query("DROP TABLE IF EXISTS ".$wpdb->prefix.$this->pp_prefix."backlink");
  }

 function addHooks() {
  /* Description: Adds the hooks */
    
  //Add JS where needed
  add_action('admin_enqueue_scripts', array($this, 'registerEnqueueAdminJS'));

  //Add CSS where needed
  add_action('admin_enqueue_scripts', array($this, 'registerEnqueueAdminCSS'));
  
  //Add pages
  add_action('admin_menu', array($this, 'registerPages'), 9999);
  
  //Listen for stuff
  add_action('init', array($this, 'do_on_backlinkClick')); //Listen for backlinkchecks
  
  //Add AJAX hooks
  add_action('wp_ajax_'.$this->pp_prefix.'backlinkManuallyPromoteBacklink', array($this, 'ajaxManuallyPromoteBacklink_callback'));
  }

 function init() {
  /* Description: This initiates the plugin and is run everytime as long as the plugin is active. */
  
  //Add default options (cannot be performed in $this->activate, because "$this->pp_config" is not available there)
  foreach($this->pp_config AS $key => $value) {
   add_option($this->pp_prefix.$key, $value);
   }
  
  //Get config-Vars from database
  foreach($this->pp_config AS $key => $value) {
   $this->pp_config[$key] = get_option($this->pp_prefix.$key);
   }
  
  //Add hooks
  $this->addHooks();
  }

 function registerPages() {
  add_menu_page($this->pp_plugin_name_read, $this->pp_plugin_name_read, 'activate_plugins', $this->pp_prefix.'menuPages', array($this, 'pageDashboard'), 'dashicons-admin-plugins', $this->getNextFreeMenuPosition('65')); 
  
  //Backlinks
  add_submenu_page($this->pp_prefix.'menuPages', 'Backlinks', 'Backlinks', 'activate_plugins', $this->pp_prefix.'menuPageBacklinks', array($this, 'pageBacklinks'));
  
  //Invisible pages
  //Configuration
  add_submenu_page(null, 'Configuration', 'Configuration', 'activate_plugins', $this->pp_prefix.'menuPageConfiguration', array($this, 'pageConfiguration'));
  //Wizard
  add_submenu_page(null, 'Wizard', 'Wizard', 'activate_plugins', $this->pp_prefix.'menuPageWizard', array($this, 'pageWizard'));
  }
    
 //CSS files
 function registerEnqueueAdminCSS() {
  /* Description: Enqueues plugin specificy CSS in the admin panel */
  
  //No versioning needed, as only one instance of this can be present at the same time
  wp_register_style($this->pp_prefix.'admin_css',
                    plugins_url().'/'.$this->pp_plugin_filefolder_name.'/plugin/css/admin.css');
  
  //Enqueue the script
  wp_enqueue_style($this->pp_prefix.'admin_css');
  
  //Enqeue generic scripts
  wp_enqueue_style('thickbox');
  wp_enqueue_style('wp-pointer');
  }

 //JS files
 function registerEnqueueAdminJS() {
  /* Description: Enqueues plugin specificy JS in the admin panel */

  //No versioning needed, as only one instance of this can be present at the same time
  wp_register_script($this->pp_prefix.'admin_js',
                     plugins_url().'/'.$this->pp_plugin_filefolder_name.'/plugin/js/admin.js',
                     array('jquery', 'thickbox', 'suggest', 'jquery-ui-datepicker', 'jquery-ui-autocomplete', 'jquery-ui-progressbar'));
  
  if(!wp_script_is($this->pp_prefix.'admin_js', 'enqueued')) {
   //Add inline script direclty BEFORE the js (so it knows, what it is)
   wp_add_inline_script($this->pp_prefix.'admin_js',
                        'var pp_this_plugin = "'.$this->pp_plugin_name.'";',
                        'before');
                        
   //Add inline script direclty AFTER the js (so it forgets everything)
   wp_add_inline_script($this->pp_prefix.'admin_js',
                        'var pp_this_plugin = "";',
                        'after');
   }
  
  //Enqueue the script
  wp_enqueue_script($this->pp_prefix.'admin_js');
  
  //Pass PHP-Variables to JS with "wp_localize_script"
  wp_localize_script($this->pp_prefix.'admin_js',
                     $this->pp_plugin_name,
                     array('ajax_url' => admin_url('admin-ajax.php'),
	                          'pp_prefix' => $this->pp_prefix,
	                          'pp_plugin_name_read' => $this->pp_plugin_name_read));
  }

 //Plugin pages
 function pageDashboard() {
  /* Description: Displays dashboard page */
  $pagename = $this->pp_prefix.'menu'.ucfirst(__FUNCTION__);
  $include = substr(strtolower(__FUNCTION__), 4);
  require(plugin_dir_path(__FILE__)."pages/".$include."_code.inc.php");
  require(plugin_dir_path(__FILE__)."pages/".$include."_output.inc.php");
  }

 function pageConfiguration() {
  /* Description: Displays config page */
  $pagename = $this->pp_prefix.'menu'.ucfirst(__FUNCTION__);
  $include = substr(strtolower(__FUNCTION__), 4);
  require(plugin_dir_path(__FILE__)."pages/".$include."_code.inc.php");
  require(plugin_dir_path(__FILE__)."pages/".$include."_output.inc.php");
  }

 function pageBacklinks() {
  /* Description: Displays the backlink page */
  $pagename = $this->pp_prefix.'menu'.ucfirst(__FUNCTION__);
  $include = substr(strtolower(__FUNCTION__), 4);
  require(plugin_dir_path(__FILE__)."pages/".$include."_code.inc.php");
  require(plugin_dir_path(__FILE__)."pages/".$include."_output.inc.php");
  }

 function pageWizard() {
  /* Description: Displays the backlink page */
  $pagename = $this->pp_prefix.'menu'.ucfirst(__FUNCTION__);
  $include = substr(strtolower(__FUNCTION__), 4);
  require(plugin_dir_path(__FILE__)."pages/".$include."_code.inc.php");
  require(plugin_dir_path(__FILE__)."pages/".$include."_output.inc.php");
  }
 
 //Lib functions
 function getPluginData($plugin_name = false) {
  /* Description: Generic function to retrieve the plugin data from the main plugin file. */
  if(!function_exists('get_plugin_data')) {
   //Without this, get_plugin_data only works in admin area
		 require_once(ABSPATH.'/wp-admin/includes/plugin.php');
		 }
		
		if($plugin_name == false) {
 		return false;
		 } else {
 		return get_plugin_data(WP_PLUGIN_DIR.'/'.$plugin_name.'/'.$plugin_name.'.php',
                          $markup = false,
                          $translate = false);
   }
  }

 function getNextFreeMenuPosition($startposition) {
  /* Description: Get the next free menu position in Wordpress
    
     Thank you for the idea: http://blog.happyplugins.com/coding-how-to-avoid-conflict-menu-position-in-wordpress/
 	   - Important: Call add_menu hook as late as possible, so other plugins
 	                that do not support this feature won't overwrite our nav entry
 	                add_action('admin_menu', 'my_add_function', 9999);
  
  */
  $startposition = strval($startposition);
  
  foreach($GLOBALS['menu'] as $key => $menu) {
  	$positions[] = $key;
   }
  
  if(!in_array($startposition, $positions)) {
  	//Position is free => return desired position
   return $startposition;
   } else {
   /* Position is taken ($key contains the position) => increment until next free position */
   while(in_array($startposition, $positions)) {
    $startposition += 0.00001;
    //
    $startposition = strval($startposition);
    }
   return $startposition;
   }
  }

 function getPagingStatement($page = 1, $show_results = 50) {
	 if(isset($page)
	    AND is_numeric($page)
	    AND ($page > 0)) {
	  $page = ceil($page);   
	  } else {
	  $page = 1; 
	  }
	
	 if(isset($show_results)
	    AND is_numeric($show_results)
	    AND ($show_results > 0)) {
	  $show_results = ceil($show_results);   
	  } else {
	  $show_results = 50; 
	  }
	 
	 $page = $page - 1;
	 
	 //Wenn $page = 0 => $start_results = 0
	 $start_results = $page * $show_results;
	 
	 return " LIMIT ".$start_results.", ".$show_results;
	 }

 function getPagingOutput($currentpage, $paging_base_link = "admin.php", $paging_variables, $paging_total_results, $paging_show_results = 50) {
	 //Define current page
	 if(isset($currentpage)
	    AND is_numeric($currentpage)
	    AND ($currentpage > 0)) {
	  $paging_currentpage = round($currentpage);
	  } else {
	  $paging_currentpage = 1;
	  }
	
		if(isset($paging_show_results)
	    AND is_numeric($paging_show_results)
	    AND ($paging_show_results > 0)) {
	  $paging_show_results = ceil($paging_show_results);   
	  } else {
	  $paging_show_results = 50; 
	  }

	 //Number of result pages
	 $paging_result_pages = ceil($paging_total_results / $paging_show_results);
	 
	 //Adjust paging
	 if($paging_currentpage > $paging_result_pages) $paging_currentpage = $paging_result_pages;
	 if($paging_currentpage < 1) $paging_currentpage = 1;
	
	 //Calculate query string
	 if(isset($paging_variables)
	    AND is_array($paging_variables)) {
	  $i = 0;
	  foreach($paging_variables AS $this_key => $this_value) {
	   if($i == 0) {
	    $paging_base_link .= "?";
	    } else {
	    $paging_base_link .= "&";
	    }
	   
	   $paging_base_link .= htmlspecialchars($this_key)."=".htmlspecialchars($this_value);
	 
	   $i++;
	   }
	  
	  $paging_base_link .= "&_PAGE=";
	  } else {
	  $paging_base_link .= "?_PAGE=";
	  }
	  
  if($paging_result_pages > 1) {
	  /* Only show when more than one page */ ?>
	  
	  <form action="<?php echo $paging_base_link; ?>" method="get">
	   <?php if(isset($paging_variables)
	            AND is_array($paging_variables)) {
	          foreach($paging_variables AS $this_key => $this_value) { ?>
		   <input type="hidden" name="<?php echo htmlspecialchars($this_key); ?>" value="<?php echo htmlspecialchars($this_value); ?>">
		  <?php   }
		         } ?>
	   
	   <div class="tablenav">
	 		 <div class="tablenav-pages">
	 		 	<span class="displaying-num"><?php echo $paging_total_results; ?> results</span>
	 		 	<a class="first-page<?php if($paging_currentpage <= 1) echo " disabled"; ?>" title="Go to the first page" href="<?php echo htmlspecialchars($paging_base_link); ?>1">«</a> 
	 		 	<a class="prev-page<?php if($paging_currentpage <= 1) echo " disabled"; ?>" title="Go to the previous page" href="<?php echo htmlspecialchars($paging_base_link); ?><?php if($paging_currentpage > 2) { echo ($paging_currentpage - 1); } else { echo 1; } ?>">‹</a> 
	 		 	<span class="paging-input"><input class="current-page" title="Current page" type="text" name="_PAGE" value="<?php echo $paging_currentpage; ?>" size="1"> of <span class="total-pages"><?php echo $paging_result_pages; ?></span></span> 
	 		 	<a class="next-page<?php if($paging_currentpage > ($paging_result_pages-1)) echo " disabled"; ?>" title="Go to the next page" href="<?php echo htmlspecialchars($paging_base_link); ?><?php if($paging_currentpage < $paging_result_pages) { echo ($paging_currentpage + 1); } else { echo $paging_currentpage; } ?>">›</a> 
	 		 	<a class="last-page<?php if($paging_currentpage > ($paging_result_pages-1)) echo " disabled"; ?>" title="Go to the last page" href="<?php echo htmlspecialchars($paging_base_link); ?><?php echo ($paging_result_pages); ?>">»</a>
	 		 </div>
	 	 </div>
	 	</form>
		<?php  
		 }
	 }

 function updateOptions($option_vars, $option_values_new) {
  /* Description: Updates the options of a plugin. */
  
  //Set default return to false
  $option_changed = false;
  
  //Read option's values from database
  foreach($option_vars AS $key => $regexp) {
   $option[$key] = get_option($key);
   }
   
  //Check if the form was submitted options
  if(isset($option_values_new)
     AND is_array($option_values_new)
     AND ($option != false)) {
  
   while(list($key, $value) = each($option_values_new)) {
    //Remove slashes for comparison
    $value = stripslashes($value);
    
    if($option[$key] == $value) {
     //wenn bisheriger wert gleich dem neuen wert
     $option_changed[$key] = 'SAME';
     } else {
     //wenn bisheriger wert alt => updaten in datenbank, updaten in session
     if(isset($option_vars[$key])
        AND ($option_vars[$key] != false)
        AND ((is_callable($option_vars[$key])
              AND ($option_vars[$key]($value) == true))
             OR ((is_string($option_vars[$key])
                 AND (preg_match($option_vars[$key], $value) != false))))) {
      update_option($key, $value);
      $option_changed[$key] = 'CHANGED';
      $option[$key] = $value;
      } else {
      $option_changed[$key] = 'ERROR';
      }
     }
    }
   }
  
  return $option_changed; 
  }
 
 //URL functions
 function getServerProtocol() {
  /* Description: Gets current protocol. */
  if(isset($_SERVER['HTTPS'])
       AND (($_SERVER['HTTPS'] == 'on')
            OR ($_SERVER['HTTPS'] == 1))
       OR (isset($_SERVER['HTTP_X_FORWARDED_PROTO'])
           AND ($_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https'))) {
   return 'https://';
   } else {
   return 'http://';
   }
  }

 function getCurrentURL() {
  /* Description: Gets current URL. */
  return self::getServerProtocol().$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];
  }

 function isBlogURL($url) {
  /* Description: Checks whether URL is an internal link. */
  if(substr($url, 0, strlen(get_site_url())) == get_site_url()) {
   return true;
   } else {
   return false;
   }
  }
 
 function isValidURLSyntax($url) {
  /* Description: Checks wether URL has a valid syntax. */
  if(phpversion() >= '5.2.0') {
   //filter_var is only available above php 5.2. ...
   if(filter_var($url, FILTER_VALIDATE_URL)) {
    $valid_url = true;
    } else {
    $valid_url = false;
    }
   } else {
   //Check with regexp
   if(preg_match('#\b((http://|https://)?((([a-z]|[0-9]|ä|ö|ü|Ä|Ö|Ü)([a-z]|[0-9]|ä|ö|ü|Ä|Ö|Ü|\-){0,61}([a-z]|[0-9]|ä|ö|ü|Ä|Ö)\.))*([a-z]|[0-9]|ä|ö|ü|Ä|Ö)([a-z]|[0-9]|ä|ö|ü|Ä|Ö|\-){0,61}([a-z]|[0-9]|ä|ö|ü|Ä|Ö)\.(af|ax|al|dz|as|ad|ao|ai|aq|ag|ar|am|aw|au|at|az|bs|bh|bd|bb|by|be|bz|bj|bm|bt|bo|ba|bw|bv|br|io|bn|bg|bf|bi|kh|cm|ca|cv|ky|cf|td|cl|cn|cx|cc|co|km|cg|cd|ck|cr|ci|hr|cu|cy|cz|dk|dj|dm|do|ec|eg|sv|gq|er|ee|et|fk|fo|fj|fi|fr|gf|pf|tf|ga|gm|ge|de|gh|gi|gr|gl|gd|gp|gu|gt|gg|gn|gw|gy|ht|hm|va|hn|hk|hu|is|in|id|ir|iq|ie|im|il|it|jm|jp|je|jo|kz|ke|ki|kp|kr|kw|kg|la|lv|lb|ls|lr|ly|li|lt|lu|mo|mk|mg|mw|my|mv|ml|mt|mh|mq|mr|mu|yt|mx|fm|md|mc|mn|ms|ma|mz|mm|na|nr|np|nl|an|nc|nz|ni|ne|ng|nu|nf|mp|no|om|pk|pw|ps|pa|pg|py|pe|ph|pn|pl|pt|pr|qa|re|ro|ru|rw|sh|kn|lc|pm|vc|ws|sm|st|sa|sn|cs|sc|sl|sg|sk|si|sb|so|za|gs|es|lk|sd|sr|sj|sz|se|ch|sy|tw|tj|tz|th|tl|tg|tk|to|tt|tn|tr|tm|tc|tv|ug|ua|ae|gb|us|um|uy|uz|vu|ve|vn|vg|vi|wf|eh|ye|zm|zw|com|edu|gov|int|mil|net|org|biz|info|name|pro|aero|coop|museum|arpa|eu))/?\b#sm', $url) != false) {
    $valid_url = true;
    } else {
    $valid_url = false; 
    }
   }
  return $valid_url;
  }

 function isSearchEngineURL($referrer_url = false) {
  /* Description: Checks if URL is a search engine URL */
  if($referrer_url == false) $referrer_url = $_SERVER['HTTP_REFERER'];
  
  $search_engine_urls = array('msn.',
                              'yahoo.',
                              'live.',
                              'aol.',
                              'ask.',
                              'altavista.',
                              'netscape.',
                              'alltheweb.',
                              'lycos.',
                              'cuil.',
                              'gutefrage.net',
                              'google.',
                              'bing.',
                              'web.info.com',
                              'search.',
                              'del.icio.us/search',
                  	           'soso.com',
                  	           '/search/',
                  	           '.yahoo.');
 	
  foreach($search_engine_urls AS $this_search_engine_url) {
   if(strpos($referrer_url, $this_search_engine_url) !== false) return true;
   }
  return false;	
  }

 function unparseURL($parsed_url) {
  /* Description: Creates an URL out of an array generated with parse_url */
  //Thanks to: thomas@gielfeldt.com on http://php.net/manual/en/function.parse-url.php
  
  $scheme = isset($parsed_url['scheme']) ? $parsed_url['scheme'] . '://' : ''; 
  $host = isset($parsed_url['host']) ? $parsed_url['host'] : ''; 
  $port = isset($parsed_url['port']) ? ':' . $parsed_url['port'] : ''; 
  $user = isset($parsed_url['user']) ? $parsed_url['user'] : ''; 
  $pass = isset($parsed_url['pass']) ? ':' . $parsed_url['pass']  : ''; 
  $pass = ($user || $pass) ? "$pass@" : ''; 
  $path = isset($parsed_url['path']) ? $parsed_url['path'] : ''; 
  $query = isset($parsed_url['query']) ? '?' . $parsed_url['query'] : ''; 
  $fragment = isset($parsed_url['fragment']) ? '#' . $parsed_url['fragment'] : ''; 
  
  return "$scheme$user$pass$host$port$path$query$fragment"; 
  } 
 
 function removeHashFromURL($url) {
  /* Description: Removes the Hashtag and everything after it from an URL */
  $parsed_url = parse_url($url);
  if(isset($parsed_url['fragment'])) {
   unset($parsed_url['fragment']);
   }
  return self::unparseURL($parsed_url);
  }

 //Common functions with wplinkbuilder-pro
 public function promoteBacklink($url_promoted, $timeout = 5) {
  /* Description: Contact the backlink webservice to push the backlinks to indexing services. */
  
  if($this->pp_config['enable_backlink_promotion'] == 'Y') {
   //Only do this, if the user accepted to activate the promotion along with privacy policy!
   
   if($this->isValidURLSyntax($url_promoted)) {
    //Only check if valid URL
    
    if(ini_get('allow_url_fopen')) {
     $postdata = http_build_query(array('action' => $this->pp_constant['promotion_api_name'].'_backlinkPromotion',
                                        $this->pp_constant['promotion_api_name'].'_data' => array('url_promoted' => $url_promoted)));
                                                                      
     //Create options
     $options = array('http' => array('header'  => 'Content-type: application/x-www-form-urlencoded',
                                      'timeout' => $timeout,
                                      'method' => 'POST',
                                      'content' => $postdata));
                                     
     //Create stream
     $backlink_promotion_context = stream_context_create($options);
    
     //Load url
     $backlink_promotion_content = file_get_contents($this->pp_constant['promotion_api_endpoint'], false, $backlink_promotion_context); 
    
     //check if url could be loaded
     if($backlink_promotion_content == false) {
      //url could not be loaded
      $return_var = 'ERROR_LOAD';
      } else {
      //create array, if url could be loaded
      $backlink_promotion_content_decoded = @json_decode($backlink_promotion_content);
      
      if(is_object($backlink_promotion_content_decoded)
         AND ($backlink_promotion_content_decoded->status == 'Y')) {
       //Return OK
       $return_var = 'OK';
       } else {
       //Result was false
       $return_var = 'ERROR_RESULT';
       }
      }
     } else {
     //FOPEN not activated
     $return_var = 'ERROR_FOPEN';
     } 
    } else {
    //In case of wrong syntax
    $return_var = 'ERROR_SYNTAX';
    }
   } else {
   //In case this is not activated in the config
   $return_var = 'ERROR_CONFIG';
   }

  //Return the result
  return $return_var;
  }
  
 public function backlinkExists($url_referrer, $url_linkto) {
  /* Description: Check if the backlink already exists in the database */
  global $wpdb;
  
  $backlink_query = "SELECT ".$wpdb->prefix.$this->pp_prefix."backlink.*
                     FROM ".$wpdb->prefix.$this->pp_prefix."backlink
                     WHERE (".$wpdb->prefix.$this->pp_prefix."backlink.url_referrer = '".$url_referrer."')
                           AND (".$wpdb->prefix.$this->pp_prefix."backlink.url_linkto = '".$url_linkto."')
                     LIMIT 0, 1";
  
  $backlink_result = $wpdb->get_results($backlink_query);
  if($backlink_result != false) {
   //There is at least one backlink with a target of url_linkto
   return true;
   } else {
   return false;
	  }
 	}

 public function promotionExists($url_referrer) {
  /* Description: Check if the the referrer already exists (promoted) in the backlink database */
  global $wpdb;
  
  $promotion_query = "SELECT ".$wpdb->prefix.$this->pp_prefix."backlink.*
                     FROM ".$wpdb->prefix.$this->pp_prefix."backlink
                     WHERE (".$wpdb->prefix.$this->pp_prefix."backlink.url_referrer = '".$url_referrer."')
                           AND (".$wpdb->prefix.$this->pp_prefix."backlink.status_promoted = 'Y')
                     LIMIT 0, 1";
  
  $promotion_result = $wpdb->get_results($promotion_query);
  if($promotion_result != false) {
   //There is at least one referrer in the database that was already promoted
   return true;
   } else {
   return false;
	  }
 	}
 
 public function manuallyPromoteBacklink($backlink_id) {
  /* Description: Loads a backlink from the database and promotes it */
  global $wpdb;
  
  $backlink_query = "SELECT ".$wpdb->prefix.$this->pp_prefix."backlink.*
                     FROM ".$wpdb->prefix.$this->pp_prefix."backlink
                     WHERE (".$wpdb->prefix.$this->pp_prefix."backlink.backlink_id = '".$backlink_id."')
                     LIMIT 0, 1";
  
  $backlink_result = $wpdb->get_results($backlink_query);
  if($backlink_result != false) {
   //Backlink is in the database
   
   $promote_result = $this->promoteBacklink($backlink_result[0]->url_referrer, $this->pp_config['promotion_timeout']);
   
   if($promote_result == 'OK') {
    //Update BL in database
    $wpdb->update($wpdb->prefix.$this->pp_prefix."backlink",
                  array('status_promoted' => 'Y'),
                  array('url_referrer' => $backlink_result[0]->url_referrer));
    
    return true;
    } else {
    //Promotion failed
    return false;
    } 
   } else {
   return false;
	  }
 	}
   
 //Hooked functions
 function do_on_backlinkClick() {
  /* Description: Listens for URL-calls and log them, if configured that way */
  global $wpdb;
  
  //Remove the hash as it makes no difference when crawling
  $url_referrer = $this->removeHashFromURL($_SERVER['HTTP_REFERER']);
  $url_linkto = $this->getCurrentURL();
  
  //Check if valid URL to check for backlinks
  if($this->isValidURLSyntax($url_referrer)
     AND !$this->isBlogURL($url_referrer)
     AND !$this->isSearchEngineURL($url_referrer)
     AND !$this->backlinkExists($url_referrer, $url_linkto)) {
   
   if($this->promotionExists($url_referrer)) {
    $status_promoted = 'Y';
    } else {
    $status_promoted = 'N';
    }
    
   //Insert BL into database
   $backlink_insert_array = array('url_referrer' => $url_referrer,
                                  'url_linkto' => $url_linkto,
                                  'date_detected' => date('Y-m-d H:i:s'),
                                  'status_promoted' => $status_promoted);
   
   $wpdb->insert($wpdb->prefix.$this->pp_prefix."backlink", $backlink_insert_array);
   $backlink_id = $wpdb->insert_id;
   
   if($backlink_id != false) {
    if($this->pp_config['auto_backlink_promotion'] == 'Y') {
     $this->manuallyPromoteBacklink($backlink_id);
     } 
    } else {
    //Backlink insertion failed
    }
   } else {
   //Backlink validation failed or already exists... 
   }
  }

 //Ajax functions
 public function ajaxManuallyPromoteBacklink_callback() {
  /* Description: Ajax wrapper for manual promotion of backlinks. */
  if(wp_verify_nonce($_REQUEST['_wpnonce'], $this->pp_prefix.'ajaxManuallyPromoteBacklink') != false) {
   $promotion_status = $this->manuallyPromoteBacklink($_POST[$this->pp_prefix.'data']['backlink_id']);
   } else {
   $promotion_status = false;
   } 
   
  echo json_encode($promotion_status);
  
  wp_die(); //This is required to terminate immediately and return a proper response
  } 
 }
   
?>